package emi.indo.mobile.firebase.authentication;

import android.app.Activity;
import android.content.Intent;


import androidx.annotation.NonNull;

import com.google.android.gms.auth.api.signin.GoogleSignIn;
import com.google.android.gms.auth.api.signin.GoogleSignInAccount;
import com.google.android.gms.auth.api.signin.GoogleSignInClient;
import com.google.android.gms.auth.api.signin.GoogleSignInOptions;
import com.google.android.gms.common.api.ApiException;
import com.google.android.gms.tasks.Task;
import com.google.firebase.FirebaseException;
import com.google.firebase.auth.AuthCredential;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.auth.GoogleAuthProvider;
import com.google.firebase.auth.PhoneAuthCredential;
import com.google.firebase.auth.PhoneAuthOptions;
import com.google.firebase.auth.PhoneAuthProvider;

import org.apache.cordova.CallbackContext;
import org.apache.cordova.CordovaInterface;
import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.CordovaWebView;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.concurrent.TimeUnit;

/**
 * This class echoes a string called from JavaScript.
 */
public class MobileFirebaseAuthentication extends CordovaPlugin {

    private CallbackContext PUBLIC_CALLBACKS = null;

    private FirebaseAuth mAuth;
    protected Activity mActivity;
    String codeByFirebase;


    private static final int RC_SIGN_IN = 9001;

    @Override
    public void initialize(CordovaInterface cordova, CordovaWebView webView) {
        super.initialize(cordova, webView);
        mAuth = FirebaseAuth.getInstance();
        mActivity = this.cordova.getActivity();
    }


    @Override
    public boolean execute(String action, JSONArray args, CallbackContext callbackContext) throws JSONException {
        PUBLIC_CALLBACKS = callbackContext;
        if (action.equals("getCurrentUser")) {

                getCurrentUser(callbackContext);

            return true;

        }
        if (action.equals("createUserWithEmailAndPassword")) {

           final String email = args.getString(0);
           final String password = args.getString(1);

                createUserWithEmailAndPassword(email, password);
                return true;
        }
        if (action.equals("signInWithEmailAndPassword")) {

           final String email = args.getString(0);
           final String password = args.getString(1);
            signInWithEmailAndPassword(email, password, callbackContext);
            return true;

        }
        if (action.equals("signInWithGoogle")) {
            final String clientID = args.getString(0);
            cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {

                    login(clientID);
                }
            });

            return true;
        }


        if (action.equals("signInAnonymously")) {

            signInAnonymously();
            return true;

        }
        if (action.equals("signInWithPhoneAuth")) {

          final String phoneNumber = args.getString(0);
           final long setTimeout = args.getLong(1);
            mAuth = FirebaseAuth.getInstance();
            sendVerificationCodeToUser(phoneNumber, setTimeout, callbackContext);
           return true;

        }
        if (action.equals("verifyCode")) {

         final String code = args.getString(0);
          verifyCode(code);
          return true;

        }
        if (action.equals("signOut")) {

            FirebaseUser user = FirebaseAuth.getInstance().getCurrentUser();
            if (user != null) {

                try {
                    FirebaseAuth.getInstance().signOut();
                    callbackContext.success();
                } catch (NullPointerException e) {
                    callbackContext.error(e.toString());
                }

            } else {

                callbackContext.error("User is Not signed in");
            }
            return true;
        }





        return false;
    }


    private void login(String clientID) {
        GoogleSignInOptions gso = new GoogleSignInOptions.Builder(GoogleSignInOptions.DEFAULT_SIGN_IN)
                .requestIdToken(clientID)
                .requestEmail()
                .build();

        GoogleSignInClient googleSignInClient = GoogleSignIn.getClient(cordova.getActivity(), gso);
        Intent signInIntent = googleSignInClient.getSignInIntent();

        if (signInIntent != null) {
            cordova.startActivityForResult(this, signInIntent, RC_SIGN_IN);
        } else {
            //Log.e("GoogleSignIn", "Sign-in intent is null");
            PUBLIC_CALLBACKS.error("Sign-in intent is null");
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == RC_SIGN_IN) {
            Task<GoogleSignInAccount> task = GoogleSignIn.getSignedInAccountFromIntent(data);
            try {
                GoogleSignInAccount account = task.getResult(ApiException.class);
                if (account != null) {
                    firebaseAuthWithGoogle(account);
                }
            } catch (ApiException e) {
               // Log.e("GoogleSignIn", "Login gagal: " + e.getStatusCode() + " - " + e.getMessage());
                PUBLIC_CALLBACKS.error("Login gagal: " + e.getMessage());
            }
        }
    }

    private void firebaseAuthWithGoogle(GoogleSignInAccount acct) {
        AuthCredential credential = GoogleAuthProvider.getCredential(acct.getIdToken(), null);
        mAuth.signInWithCredential(credential)
                .addOnCompleteListener(cordova.getActivity(), task -> {
                    if (task.isSuccessful()) {
                        FirebaseUser user = mAuth.getCurrentUser();
                        if (user != null) {
                            getCurrentUser(PUBLIC_CALLBACKS);
                        }
                    } else {
                        String errorMsg = task.getException() != null ? task.getException().getMessage() : "Unknown error";
                      //  Log.e("GoogleSignIn", "Login gagal: " + errorMsg);
                        PUBLIC_CALLBACKS.error("Login gagal: " + errorMsg);
                    }
                });
    }



    private void signInAnonymously() {

        mAuth.signInAnonymously()
                .addOnCompleteListener(mActivity, task -> {
                    if (task.isSuccessful()) {
                        getCurrentUser(PUBLIC_CALLBACKS);
                    } else {
                        PUBLIC_CALLBACKS.error(String.valueOf(task.getException()));
                    }
                });

    }

    private void getCurrentUser(CallbackContext callbackContext) {

        FirebaseUser user = FirebaseAuth.getInstance().getCurrentUser();
            if (user != null) {

                JSONObject result = new JSONObject();

                try {
                    result.put("uid", user.getUid());
                    result.put("displayName", user.getDisplayName());
                    result.put("email", user.getEmail());
                    result.put("phoneNumber", user.getPhoneNumber());
                    result.put("photoURL", user.getPhotoUrl());
                    result.put("providerId", user.getProviderId());
                    result.put("emailVerified", user.isEmailVerified());
                    callbackContext.success(result);
                } catch (Exception e) {
                    callbackContext.error(e.toString());
                }
        } else {
                callbackContext.error("No user is signed in" );
        }

    }

    private void signInWithEmailAndPassword(String email, String password, CallbackContext callbackContext) {

        mAuth.signInWithEmailAndPassword(email, password)
                .addOnCompleteListener(mActivity, task -> {
                    if (task.isSuccessful()) {
                        getCurrentUser(callbackContext);
                    } else {
                        callbackContext.error(String.valueOf(task.getException()));
                    }
                });

    }

    private void createUserWithEmailAndPassword(String email, String password) {

        mAuth.createUserWithEmailAndPassword(email, password)
                .addOnCompleteListener(mActivity, task -> {
                    if (task.isSuccessful()) {
                        getCurrentUser(PUBLIC_CALLBACKS);
                    } else {
                        PUBLIC_CALLBACKS.error(String.valueOf(task.getException()));

                    }
                });

    }

    private void sendVerificationCodeToUser(String phoneNumber, Long setTimeout, final CallbackContext callbackContext) {

            PhoneAuthOptions options =
                    PhoneAuthOptions.newBuilder(mAuth)
                            .setPhoneNumber(phoneNumber)
                            .setTimeout(setTimeout, TimeUnit.SECONDS)
                            .setActivity(mActivity)
                            .setCallbacks(new PhoneAuthProvider.OnVerificationStateChangedCallbacks() {
                                @Override
                                public void onVerificationCompleted(@NonNull PhoneAuthCredential credential) {
                                    PUBLIC_CALLBACKS.success(credential.toString());
                                }

                                @Override
                                public void onVerificationFailed(@NonNull FirebaseException e) {
                                    callbackContext.error( e.getMessage());
                                }
                                @Override
                                public void onCodeSent(@NonNull String verificationId, @NonNull PhoneAuthProvider.ForceResendingToken token) {
                                    super.onCodeSent(verificationId, token);
                                    codeByFirebase = verificationId;
                                }
                            })
                            .build();

                            PhoneAuthProvider.verifyPhoneNumber(options);

                         }


    private void verifyCode(String code) {
        PhoneAuthCredential credential = PhoneAuthProvider.getCredential(codeByFirebase, code);
        signInWithPhoneAuthCredential(credential);
    }

    private void signInWithPhoneAuthCredential(PhoneAuthCredential credential) {

        mAuth.signInWithCredential(credential)
                .addOnCompleteListener(mActivity, task -> {
                    if (task.isSuccessful()) {
                       PUBLIC_CALLBACKS.success(task.toString());
                    } else {
                      PUBLIC_CALLBACKS.error("Verification Error");
                    }
                });
    }


}
